@@ -1,6 +1,6 @@
 /* mpc_asin -- arcsine of a complex number.
 
-Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014, 2020 INRIA
+Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014, 2020, 2022 INRIA
 
 This file is part of GNU MPC.
 
@@ -154,20 +154,20 @@ mpc_asin_series (mpc_srcptr rop, mpc_ptr s, mpc_srcptr z, mpc_rnd_t rnd)
   /* ulp(Re(s)) = 2^(ex+1-p) */
   err = 0;
   /* invariant: the error will be kx*2^err */
-  if (ex+1 > e) /* divide kx by 2^(ex+1-e) */
-    while (ex+1 > e)
+  if (ex + 1 > e) /* divide kx by 2^(ex+1-e) */
+    while (ex + 1 > e)
       {
         kx = (kx + 1) / 2;
-        ex --;
+        ex--;
       }
   else /* multiply the error by 2^(e-(ex+1)), thus add e-(ex+1) to err */
-    err += e - (ex+1);
+    err += e - (ex + 1);
   /* now the rounding error is bounded by kx*2^err*ulp(Re(s)), add the
      mathematical error which is bounded by ulp(Re(s)): the first neglected
      term is less than 1/2*ulp(Re(s)), and each term decreases by at least
      a factor 2, since |z^2| <= 1/2. */
-  kx ++;
-  for (; kx > 2; err ++, kx = (kx + 1) / 2);
+  kx++;
+  for (; kx > 2; err++, kx = (kx + 1) / 2);
   /* can we round Re(s) with error less than 2^(EXP(Re(s))-err) ? */
   if (!mpfr_can_round (mpc_realref (s), p - err, MPFR_RNDN, MPFR_RNDZ,
                        mpfr_get_prec (mpc_realref (rop)) +
@@ -181,86 +181,111 @@ mpc_asin_series (mpc_srcptr rop, mpc_ptr s, mpc_srcptr z, mpc_rnd_t rnd)
      (see algorithms.tex) */
   e = mpfr_get_exp (mpc_imagref (z));
   /* ulp(Im(s)) = 2^(ey+1-p) */
-  if (ey+1 > e) /* divide ky by 2^(ey+1-e) */
-    while (ey+1 > e)
+  if (ey + 1 > e) /* divide ky by 2^(ey+1-e) */
+    while (ey + 1 > e)
       {
         ky = (ky + 1) / 2;
-        ey --;
+        ey--;
       }
   else /* multiply ky by 2^(e-(ey+1)) */
-    ky <<= e - (ey+1);
+    ky <<= e - (ey + 1);
   /* now the rounding error is bounded by ky*ulp(Im(s)), add the
      mathematical error which is bounded by ulp(Im(s)): the first neglected
      term is less than 1/2*ulp(Im(s)), and each term decreases by at least
      a factor 2, since |z^2| <= 1/2. */
-  ky ++;
-  for (err = 0; ky > 2; err ++, ky = (ky + 1) / 2);
+  ky++;
+  for (err = 0; ky > 2; err++, ky = (ky + 1) / 2);
   /* can we round Im(s) with error less than 2^(EXP(Im(s))-err) ? */
   return mpfr_can_round (mpc_imagref (s), p - err, MPFR_RNDN, MPFR_RNDZ,
                          mpfr_get_prec (mpc_imagref (rop)) +
                          (MPC_RND_IM(rnd) == MPFR_RNDN));
 }
 
-/* Put in s an approximation of asin(z) for |Re(z)| < 1 and tiny Im(y)
-   (see algorithms.tex). Assume z = x + i*y:
-   |Re(asin(z)) - asin(x)| <= Pi*beta^2/(1-beta)
-   |Im(asin(z)) - y/sqrt(1-x^2)| <= Pi/2*beta^2/(1-beta)
-   where beta = |y|/(1-|x|).
-   We assume |x| >= 1/2 > |y| here, thus beta < 1, and the bounds
-   simplify to 16y^2.
-   Assume Re(s) and Im(s) have the same precision.
-   Return non-zero if we can get the correct result by rounding s:
-   mpc_set (rop, s, ...) */
-static int
-mpc_asin_tiny (mpc_srcptr rop, mpc_ptr s, mpc_srcptr z, mpc_rnd_t rnd)
+
+static int /* bool */
+asin_taylor1 (int *inex, mpc_ptr rop, mpc_srcptr z, mpc_rnd_t rnd)
+   /* Write z = x + i*y and assume |x| < 1/2 and |y| < 1/4, that is,
+      Exp (x) <= -1 and Exp (y) <= -2; this also implies |z| < 1.
+      The function computes the Taylor series of order 1 around x
+         asin (z) \approx asin (x) + i * y / sqrt (1 - x^2)
+      with error term bounded above by Pi/2 * beta^2 / (1 - beta)
+      where beta = |y| / (1 - |x|), see algorithms.tex.
+      If the result can be rounded in direction rnd to rop, the value is
+      stored in rop, the inexact value is stored in inex, and true is
+      returned; otherwise rop and inex are not changed, and false
+      is returned. */
 {
-  mpfr_exp_t ey, e1, e2, es;
-  mpfr_prec_t p = mpfr_get_prec (mpc_realref (s)), err;
+   mpfr_exp_t ex, ey, es, err;
+   mpfr_prec_t prec_re, prec_im, prec;
+   mpfr_srcptr x, y;
+   mpfr_t s, t;
+   int inex_re, inex_im, ok;
 
-  ey = mpfr_get_exp (mpc_imagref (z));
+   /* We have asin (x) ~ x,
+      |y| <= |y| / sqrt (1 - x^2) <= sqrt (4/3) * |y|,
+      beta <= 2 * |y| < 1/2, 1 / 1 - beta < 2 and the error term is bounded
+      above by 4 * Pi * |y|^2 < 13 * |y|^2 < 16 * |y|^2.
+      So to have a chance to round the imaginary part, we need roughly
+      log_2 (error term) \approx 2 * Exp (y) + 4
+      <= log_2 (ulp (y)) \approx Exp (y) - prec (imag (rop)), or
+      Exp (y) <= -prec (imag (rop)) - 4.
+      For the real part, we need
+      2 * Exp (y) + 4 <= Exp (x) - prec (real (rop)), or
+      Exp (x) >= 2 * Exp (y) + 4 + prec (real (rop)).
+      Check this first. */
+   x = mpc_realref (z);
+   y = mpc_imagref (z);
+   ex = mpfr_get_exp (x);
+   ey = mpfr_get_exp (y);
+   prec_re = mpfr_get_prec (mpc_realref (rop));
+   prec_im = mpfr_get_prec (mpc_imagref (rop));
+   if (ey > - prec_im - 4 || ex < 2 * ey + 4 + prec_re)
+      return 0;
 
-  MPC_ASSERT(mpfr_get_exp (mpc_realref (z)) >= 0); /* |x| >= 1/2 */
-  MPC_ASSERT(ey < 0);  /* |y| < 1/2 */
+   /* Real part. */
+   prec = prec_re + 7;
+   mpfr_init2 (s, prec);
+   mpfr_asin (s, x, MPFR_RNDN);
+   /* The error is bounded above by 13*|y|^2 + 1/2 * 2^(Exp (s) - prec)
+      <= 2^(max (2 * Exp (y) + 5, Exp (s) - prec)). */
+   es = mpfr_get_exp (s);
+   err = MPC_MAX (2 * ey + 5, es - prec);
+   ok = mpfr_can_round (s, es - err, MPFR_RNDN, MPFR_RNDZ,
+      mpfr_get_prec (mpc_realref (rop))
+         + (MPC_RND_RE (rnd) == MPFR_RNDN));
 
-  e2 = 2 * ey + 4;
-  /* for |x| >= 0.5, |asin x| >= 0.5, thus no need to compute asin(x)
-     if 16y^2 >= 1/2 ulp(1) for the target variable */
-  if (e2 >= - (mpfr_exp_t) mpfr_get_prec (mpc_realref (rop)))
-    return 0;
+   if (ok) {
+      /* Imaginary part. */
+      prec = prec_im + 7;
+      mpfr_init2 (t, prec);
+      mpfr_mul (t, x, x, MPFR_RNDU); /* 0 < t <= 1/4, error 1 ulp */
+      mpfr_ui_sub (t, 1, t, MPFR_RNDD);
+         /* 3/4 <= t < 1, error 2 ulp, epsilon- = 0 since rounded down */
+      mpfr_sqrt (t, t, MPFR_RNDD);
+         /* error 3 ulp: propagation error stable since epsilon- = 0,
+            1 ulp for rounding; see ssec:proprealsqrt in algorithms.tex */
+      mpfr_div (t, y, t, MPFR_RNDA);
+         /* error 7 ulp: since denominator rounded down, previous error
+            multiplied by 2, 1 ulp additional rounding error */
+      ok = mpfr_can_round (t, prec - 3, MPFR_RNDA, MPFR_RNDZ,
+         mpfr_get_prec (mpc_imagref (rop))
+            + (MPC_RND_IM (rnd) == MPFR_RNDN));
 
-  /* real part */
-  mpfr_asin (mpc_realref (s), mpc_realref (z), MPFR_RNDN);
-  /* check that we can round with error < 16y^2 */
-  e1 = mpfr_get_exp (mpc_realref (s)) - p;
-  err = (e1 >= e2) ? 0 : e2 - e1;
-  if (!mpfr_can_round (mpc_realref (s), p - err, MPFR_RNDN, MPFR_RNDZ,
-                       mpfr_get_prec (mpc_realref (rop)) +
-                       (MPC_RND_RE(rnd) == MPFR_RNDN)))
-    return 0;
+      if (ok) {
+         inex_re = mpfr_set (mpc_realref (rop), s, MPC_RND_RE (rnd));
+         inex_im = mpfr_set (mpc_imagref (rop), t, MPC_RND_IM (rnd));
+         *inex = MPC_INEX (inex_re, inex_im);
+      }
 
-  /* now compute the approximate imaginary part y/sqrt(1-x^2) */
-  mpfr_sqr (mpc_imagref (s), mpc_realref (z), MPFR_RNDN);
-  /* now Im(s) approximates x^2, with 1/4 <= Im(s) <= 1 and absolute error
-     less than 2^-p */
-  mpfr_ui_sub (mpc_imagref (s), 1, mpc_imagref (s), MPFR_RNDN);
-  /* now Im(s) approximates 1-x^2, with 0 <= Im(s) <= 3/4, assuming p >= 2,
-     and absolute error less than 2^(1-p) */
-  mpfr_sqrt (mpc_imagref (s), mpc_imagref (s), MPFR_RNDN); /* sqrt(1-x^2) */
-  es = mpfr_get_exp (mpc_imagref (s));
-  /* now Im(s) approximates sqrt(1-x^2), with 0 <= Im(s) <= 7/8,
-     assuming p >= 3, and absolute error less than 2^-p + 2^(1-p)/s
-     <= 3*2^-p/s, thus relative error less than 3*2^-p/s^2.
-     Since |s| >= 2^(es-1), the relative error is less than 3*2^(-p+2-2*es) */
-  mpfr_div (mpc_imagref (s), mpc_imagref (z), mpc_imagref (s), MPFR_RNDN);
-  /* now Im(s) approximates y/sqrt(1-x^2), with relative error less than
-     (3*2^(2-2*es)+2)*2^-p. Since es <= 0, 2-2*es >= 2, thus the relative
-     error is less than 2^(4-2*es-p) */
-  err = 4 - 2 * es;
-  return mpfr_can_round (mpc_imagref (s), p - err, MPFR_RNDN, MPFR_RNDZ,
-                         mpfr_get_prec (mpc_imagref (rop)) +
-                         (MPC_RND_IM(rnd) == MPFR_RNDN));
+      mpfr_clear (t);
+   }
+
+   mpfr_clear (s);
+
+   return ok;
 }
 
+
 int
 mpc_asin (mpc_ptr rop, mpc_srcptr op, mpc_rnd_t rnd)
 {
@@ -375,6 +400,13 @@ mpc_asin (mpc_ptr rop, mpc_srcptr op, mpc_rnd_t rnd)
       return MPC_INEX (0, inex_im);
     }
 
+   /* Try special code for |x| < 1/2 and |y| < 1/4. */
+   if (mpfr_get_exp (mpc_realref (op)) <= -1
+      && mpfr_get_exp (mpc_imagref (op)) <= -2) {
+      if (asin_taylor1 (&inex, rop, op, rnd))
+         return inex;
+   }
+
   saved_emin = mpfr_get_emin ();
   saved_emax = mpfr_get_emax ();
   mpfr_set_emin (mpfr_get_emin_min ());
@@ -396,8 +428,7 @@ mpc_asin (mpc_ptr rop, mpc_srcptr op, mpc_rnd_t rnd)
     p += err - olderr; /* add extra number of lost bits in previous loop */
     olderr = err;
     p += (loop <= 2) ? mpc_ceil_log2 (p) + 3 : p / 2;
-    mpfr_set_prec (mpc_realref(z1), p);
-    mpfr_set_prec (mpc_imagref(z1), p);
+    mpc_set_prec (z1, p);
 
     /* try special code for 1+i*y with tiny y */
     if (loop == 1 && mpfr_cmp_ui (mpc_realref(op), 1) == 0 &&
@@ -410,12 +441,6 @@ mpc_asin (mpc_ptr rop, mpc_srcptr op, mpc_rnd_t rnd)
         mpc_asin_series (rop, z1, op, rnd))
       break;
 
-    /* try special code for 1/2 <= |x| < 1 and |y| < 1/2 */
-    if (mpfr_get_exp (mpc_realref (op)) == 0 &&
-        mpfr_get_exp (mpc_imagref (op)) <= -1 &&
-        mpc_asin_tiny (rop, z1, op, rnd))
-      break;
-
     /* z1 <- z^2 */
     mpc_sqr (z1, op, MPC_RNDNN);
     /* err(x) <= 1/2 ulp(x), err(y) <= 1/2 ulp(y) */
