@@ -1,6 +1,6 @@
 /* mpc.h -- Include file for mpc.
 
-Copyright (C) 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010, 2011, 2012, 2016, 2017, 2018, 2020 INRIA
+Copyright (C) 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010, 2011, 2012, 2016, 2017, 2018, 2020, 2021, 2022 INRIA
 
 This file is part of GNU MPC.
 
@@ -21,14 +21,15 @@ along with this program. If not, see http://www.gnu.org/licenses/ .
 #ifndef __MPC_H
 #define __MPC_H
 
+#include <stdint.h>
 #include "gmp.h"
 #include "mpfr.h"
 
 /* Define MPC version number */
 #define MPC_VERSION_MAJOR 1
-#define MPC_VERSION_MINOR 2
+#define MPC_VERSION_MINOR 3
 #define MPC_VERSION_PATCHLEVEL 0
-#define MPC_VERSION_STRING "1.2.0"
+#define MPC_VERSION_STRING "1.3.0"
 
 /* Macros dealing with MPC VERSION */
 #define MPC_VERSION_NUM(a,b,c) (((a) << 16L) | ((b) << 8) | (c))
@@ -79,21 +80,31 @@ typedef int mpc_rnd_t;
 #define MPC_RNDNZ MPC_RND (MPFR_RNDN,MPFR_RNDZ)
 #define MPC_RNDNU MPC_RND (MPFR_RNDN,MPFR_RNDU)
 #define MPC_RNDND MPC_RND (MPFR_RNDN,MPFR_RNDD)
+#define MPC_RNDNA MPC_RND (MPFR_RNDN,MPFR_RNDA)
 
 #define MPC_RNDZN MPC_RND (MPFR_RNDZ,MPFR_RNDN)
 #define MPC_RNDZZ MPC_RND (MPFR_RNDZ,MPFR_RNDZ)
 #define MPC_RNDZU MPC_RND (MPFR_RNDZ,MPFR_RNDU)
 #define MPC_RNDZD MPC_RND (MPFR_RNDZ,MPFR_RNDD)
+#define MPC_RNDZA MPC_RND (MPFR_RNDZ,MPFR_RNDA)
 
 #define MPC_RNDUN MPC_RND (MPFR_RNDU,MPFR_RNDN)
 #define MPC_RNDUZ MPC_RND (MPFR_RNDU,MPFR_RNDZ)
 #define MPC_RNDUU MPC_RND (MPFR_RNDU,MPFR_RNDU)
 #define MPC_RNDUD MPC_RND (MPFR_RNDU,MPFR_RNDD)
+#define MPC_RNDUA MPC_RND (MPFR_RNDU,MPFR_RNDA)
 
 #define MPC_RNDDN MPC_RND (MPFR_RNDD,MPFR_RNDN)
 #define MPC_RNDDZ MPC_RND (MPFR_RNDD,MPFR_RNDZ)
 #define MPC_RNDDU MPC_RND (MPFR_RNDD,MPFR_RNDU)
 #define MPC_RNDDD MPC_RND (MPFR_RNDD,MPFR_RNDD)
+#define MPC_RNDDA MPC_RND (MPFR_RNDD,MPFR_RNDA)
+
+#define MPC_RNDAN MPC_RND (MPFR_RNDA,MPFR_RNDN)
+#define MPC_RNDAZ MPC_RND (MPFR_RNDA,MPFR_RNDZ)
+#define MPC_RNDAU MPC_RND (MPFR_RNDA,MPFR_RNDU)
+#define MPC_RNDAD MPC_RND (MPFR_RNDA,MPFR_RNDD)
+#define MPC_RNDAA MPC_RND (MPFR_RNDA,MPFR_RNDA)
 
 
 /* Definitions of types and their semantics */
@@ -108,8 +119,28 @@ typedef __mpc_struct mpc_t[1];
 typedef __mpc_struct *mpc_ptr;
 typedef const __mpc_struct *mpc_srcptr;
 
+typedef struct {
+   int64_t mant;
+   int64_t exp;
+}
+__mpcr_struct;
+
+typedef __mpcr_struct mpcr_t [1];
+typedef __mpcr_struct *mpcr_ptr;
+typedef const __mpcr_struct *mpcr_srcptr;
+
+typedef struct {
+  mpc_t  c;
+  mpcr_t r;
+}
+__mpcb_struct;
+
+typedef __mpcb_struct mpcb_t [1];
+typedef __mpcb_struct *mpcb_ptr;
+typedef const __mpcb_struct *mpcb_srcptr;
+
 /* Support for WINDOWS DLL, see
-   http://lists.gforge.inria.fr/pipermail/mpc-discuss/2011-November/000990.html;
+   https://sympa.inria.fr/sympa/arc/mpc-discuss/2011-11/ ;
    when building the DLL, export symbols, otherwise behave as GMP           */
 #if defined (__MPC_LIBRARY_BUILD) && __GMP_LIBGMP_DLL
 #define __MPC_DECLSPEC __GMP_DECLSPEC_EXPORT
@@ -191,6 +222,7 @@ __MPC_DECLSPEC int  mpc_cmp_abs     (mpc_srcptr, mpc_srcptr);
 __MPC_DECLSPEC int  mpc_exp         (mpc_ptr, mpc_srcptr, mpc_rnd_t);
 __MPC_DECLSPEC int  mpc_log         (mpc_ptr, mpc_srcptr, mpc_rnd_t);
 __MPC_DECLSPEC int  mpc_log10       (mpc_ptr, mpc_srcptr, mpc_rnd_t);
+__MPC_DECLSPEC int  mpc_agm         (mpc_ptr, mpc_srcptr, mpc_srcptr, mpc_rnd_t);
 __MPC_DECLSPEC int  mpc_sin         (mpc_ptr, mpc_srcptr, mpc_rnd_t);
 __MPC_DECLSPEC int  mpc_cos         (mpc_ptr, mpc_srcptr, mpc_rnd_t);
 __MPC_DECLSPEC int  mpc_sin_cos     (mpc_ptr, mpc_ptr, mpc_srcptr, mpc_rnd_t, mpc_rnd_t);
@@ -237,8 +269,63 @@ __MPC_DECLSPEC long double _Complex mpc_get_ldc (mpc_srcptr, mpc_rnd_t);
 #ifdef _GMP_H_HAVE_FILE
 __MPC_DECLSPEC int mpc_inp_str    (mpc_ptr, FILE *, size_t *, int, mpc_rnd_t);
 __MPC_DECLSPEC size_t mpc_out_str (FILE *, int, size_t, mpc_srcptr, mpc_rnd_t);
+__MPC_DECLSPEC void mpcb_out_str  (FILE *f, mpcb_srcptr);
 #endif
 
+__MPC_DECLSPEC int mpcr_inf_p (mpcr_srcptr r);
+__MPC_DECLSPEC int mpcr_zero_p (mpcr_srcptr r);
+__MPC_DECLSPEC int mpcr_lt_half_p (mpcr_srcptr r);
+__MPC_DECLSPEC int mpcr_cmp (mpcr_srcptr r, mpcr_srcptr s);
+__MPC_DECLSPEC void mpcr_set_inf (mpcr_ptr r);
+__MPC_DECLSPEC void mpcr_set_zero (mpcr_ptr r);
+__MPC_DECLSPEC void mpcr_set_one (mpcr_ptr r);
+__MPC_DECLSPEC void mpcr_set (mpcr_ptr r, mpcr_srcptr s);
+__MPC_DECLSPEC void mpcr_set_ui64_2si64 (mpcr_ptr r, uint64_t mant,
+   int64_t exp);
+__MPC_DECLSPEC void mpcr_max (mpcr_ptr r, mpcr_srcptr s, mpcr_srcptr t);
+__MPC_DECLSPEC int64_t mpcr_get_exp (mpcr_srcptr r);
+__MPC_DECLSPEC void mpcr_out_str (FILE *f, mpcr_srcptr r);
+__MPC_DECLSPEC void mpcr_mul (mpcr_ptr r, mpcr_srcptr s, mpcr_srcptr t);
+__MPC_DECLSPEC void mpcr_mul_2ui (mpcr_ptr r, mpcr_srcptr s,
+   unsigned long int e);
+__MPC_DECLSPEC void mpcr_sqr (mpcr_ptr r, mpcr_srcptr s);
+__MPC_DECLSPEC void mpcr_add (mpcr_ptr r, mpcr_srcptr s, mpcr_srcptr t);
+__MPC_DECLSPEC void mpcr_sub (mpcr_ptr r, mpcr_srcptr s, mpcr_srcptr t);
+__MPC_DECLSPEC void mpcr_sub_rnd (mpcr_ptr r, mpcr_srcptr s, mpcr_srcptr t,
+   mpfr_rnd_t rnd);
+__MPC_DECLSPEC void mpcr_div (mpcr_ptr r, mpcr_srcptr s, mpcr_srcptr t);
+__MPC_DECLSPEC void mpcr_div_2ui (mpcr_ptr r, mpcr_srcptr s,
+   unsigned long int e);
+__MPC_DECLSPEC int64_t sqrt_int64 (int64_t n);
+__MPC_DECLSPEC void mpcr_sqrt (mpcr_ptr r, mpcr_srcptr s);
+__MPC_DECLSPEC void mpcr_add_rounding_error (mpcr_ptr r, mpfr_prec_t p,
+   mpfr_rnd_t rnd);
+__MPC_DECLSPEC void mpcr_c_abs_rnd (mpcr_ptr r, mpc_srcptr z,
+   mpfr_rnd_t rnd);
+
+__MPC_DECLSPEC void mpcb_init (mpcb_ptr);
+__MPC_DECLSPEC void mpcb_clear (mpcb_ptr);
+__MPC_DECLSPEC mpfr_prec_t mpcb_get_prec (mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_set_prec (mpcb_ptr, mpfr_prec_t);
+__MPC_DECLSPEC void mpcb_set (mpcb_ptr, mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_set_inf (mpcb_ptr);
+__MPC_DECLSPEC void mpcb_set_c (mpcb_ptr, mpc_srcptr, mpfr_prec_t,
+   unsigned long int, unsigned long int);
+__MPC_DECLSPEC void mpcb_set_ui_ui (mpcb_ptr, unsigned long int,
+   unsigned long int, mpfr_prec_t);
+__MPC_DECLSPEC void mpcb_neg (mpcb_ptr, mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_mul (mpcb_ptr, mpcb_srcptr, mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_sqr (mpcb_ptr, mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_pow_ui (mpcb_ptr, mpcb_srcptr, unsigned long int);
+__MPC_DECLSPEC void mpcb_add (mpcb_ptr, mpcb_srcptr, mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_sqrt (mpcb_ptr, mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_div (mpcb_ptr, mpcb_srcptr, mpcb_srcptr);
+__MPC_DECLSPEC void mpcb_div_2ui (mpcb_ptr, mpcb_srcptr, unsigned long int);
+__MPC_DECLSPEC int mpcb_can_round (mpcb_srcptr, mpfr_prec_t, mpfr_prec_t,
+   mpc_rnd_t);
+__MPC_DECLSPEC int mpcb_round (mpc_ptr, mpcb_srcptr, mpc_rnd_t);
+__MPC_DECLSPEC int mpc_eta_fund (mpc_ptr, mpc_srcptr, mpc_rnd_t);
+
 #if defined (__cplusplus)
 }
 #endif
